/*
 *  Logger4LAAS.cpp
 *  Logger4LAAS
 *
 *  Created by Benjamin Vadant on 18/02/13.
 *  Copyright 2013 LAAS/CNRS. All rights reserved.
 *
 */

#include "logger4laas.h"
#include <chrono>
#include <string>
#include <vector>
#include <algorithm>


Logger4LAAS::Logger4LAAS(std::string className):className(className) {
    std::string pathConfigFile="./L4L.config";
    parseConfigFile(pathConfigFile);

    if (periodicity > 0) {
        std::thread thr(&Logger4LAAS::threadHandling, this);
        std::swap(thr, publishThread);
    }
}

Logger4LAAS::~Logger4LAAS() {
    publishThread.join();
}

void Logger4LAAS::log(LogLevel logLevel, std::string methodName, std::string info) {
    if (active && this->logLevel <= logLevel) {
        Message newMessage(className, methodName, logLevel, info);

        messages.push_back(newMessage);

        if (periodicity == 0)
            publish();
    }
}

void Logger4LAAS::publish() {
    std::ofstream outfile;
    std::string outstring;

    if (pathSavingFile == "")
        pathSavingFile = "./test.log";
    outfile.open(pathSavingFile.c_str(), std::ios_base::app);

    for (std::vector<Message>::iterator it = messages.begin(); it != messages.end(); ++it) {
        outstring = it->toString();
        outfile << outstring;

        if (displayInTerminal)
            std::cout << outstring;
    }

    messages.clear();  // All messages have been displayed and/or saved so we clear the list

    outfile.close();
}

void Logger4LAAS::threadHandling() {
    std::this_thread::sleep_for(std::chrono::milliseconds(periodicity));
    publish();
    threadHandling();
}

void Logger4LAAS::parseConfigFile(std::string pathConfigFile) {
    std::ifstream infile;

    infile.open(pathConfigFile.c_str());

    std::string field;

    while (infile.good()) {
        infile >> field;
        if (field == "Periodicity") {
            infile >> periodicity;
        }

        infile >> field;
        if (field == "DisplayInTerminal") {
            infile >> displayInTerminal;
        }

        infile >> field;
        if (field == "LogLevel") {
            int ll;
            infile >> ll;
            logLevel = LogLevel(ll);
        }

        infile >> field;
        if (field == "PathSavingFile") {
            infile >> pathSavingFile;
        }

        infile >> field;
        if (field == "InactiveClass") {
            std::string inactiveClass;
            infile >> inactiveClass;

            std::ostringstream search;  // To ensure that class with similar name aren't desactivated
            search << "!" << className << "!";

            active = (inactiveClass.find(search.str()) == std::string::npos);
        }
    }
}
